﻿Shader "Shep/Shader_BossGlow"
{
	Properties
	{
		[PerRendererData] _MainTex ("Sprite Texture", 2D) = "white" {}
		_Color ("Tint", Color) = (1,1,1,1)
		_OutRingColor ("OutRingColor", Color) = (0,0,0,1)
		_InRingColor ("InRingColor", Color) = (0,0,0,1)

		_FlashColor ("FlashColor", Color) = (0,0,0,0)
		_MinRingRange ("MinRingRange", Float) = 0
		_MaxRingRange ("MaxRingRange", Float) = 0
		_RingRadius("RingRadius", Float) = 0
		_GlowRange("GlowRange", Float) = 0

		[MaterialToggle] PixelSnap ("Pixel snap", Float) = 0
	}

	SubShader
	{
		Tags
		{ 
			"Queue"="Transparent" 
			"IgnoreProjector"="True" 
			"RenderType"="Transparent" 
			"PreviewType"="Plane"
			"CanUseSpriteAtlas"="True"
		}

		Cull Off
		Lighting Off
		ZWrite Off
		Blend One OneMinusSrcAlpha

		Pass
		{
		CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag
			#pragma multi_compile DUMMY PIXELSNAP_ON
			#include "UnityCG.cginc"
			
			struct appdata_t
			{
				float4 vertex   : POSITION;
				float4 color    : COLOR;
				float2 texcoord : TEXCOORD0;
			};

			struct v2f
			{
				float4 vertex   : SV_POSITION;
				fixed4 color    : COLOR;
				half2 texcoord  : TEXCOORD0;
			};
			
			fixed4 _Color;

			v2f vert(appdata_t IN)
			{
				v2f OUT;
				OUT.vertex = mul(UNITY_MATRIX_MVP, IN.vertex);
				OUT.texcoord = IN.texcoord;
				OUT.color = IN.color * _Color;
				#ifdef PIXELSNAP_ON
				OUT.vertex = UnityPixelSnap (OUT.vertex);
				#endif

				return OUT;
			}

			sampler2D _MainTex;
			fixed4 _OutRingColor;
			fixed4 _InRingColor;


			float _MinRingRange;
			float _MaxRingRange;

			float _RingRadius;
			float _GlowRange;

			fixed4 frag(v2f IN) : SV_Target
			{
				float xDelta = IN.texcoord.x - 0.5;
				float yDelta = IN.texcoord.y - 0.5;

				float hyp = sqrt( xDelta * xDelta + yDelta * yDelta);

				fixed4 col = tex2D(_MainTex, IN.texcoord);
				
				//the closer we are to the radius, the more we should glow!
				float delta = abs(hyp - _RingRadius);

				if( delta <= _GlowRange)
				{
					if( hyp >= _RingRadius)
						col = _OutRingColor;
					else
						col = _InRingColor;

					col.a = 1.0 - delta/_GlowRange;
				}
				else
				{
					col.a = 0;
				}

				col.rgb *= col.a;
				return col;
			}
		ENDCG
		}
	}

	Fallback "Sprites/Default"
}

